unit DGRegister;

interface

uses
  Classes, ToolsAPI, Forms, Windows, Messages, SysUtils, Menus, DGOptionsForm;

{$I DGExperts.inc}

type
  TDGExperts = class(TNotifierObject)
  protected
    procedure OnOptionClickMenu(Sender: TObject);
  public
    constructor Create;
    destructor Destroy; override;
  end;

  TDGIDENotifier = class(TNotifierObject, IOTANotifier, IOTAIDENotifier)
  private
    // IOTAIDENotifier
    procedure FileNotification(NotifyCode: TOTAFileNotification; const FileName: string; var Cancel: Boolean);
    procedure BeforeCompile(const Project: IOTAProject; var Cancel: Boolean); overload;
    procedure AfterCompile(Succeeded: Boolean); overload;
  end;

  TDgSourceEditorNotifier = class(TNotifierObject, IOTANotifier, IOTAEditorNotifier)
  private
    FEditor: IOTASourceEditor;
    FIndex: Integer;
    { IOTANotifier }
    procedure Destroyed;
    { IOTAEditorNotifier }
    procedure ViewActivated(const View: IOTAEditView);
    procedure ViewNotification(const View: IOTAEditView; Operation: TOperation);
  public
    constructor Create(AEditor: IOTASourceEditor);
    destructor Destroy; override;
  end;

procedure Register;

var
  Form: TfrmOptions;
  DGExperts: TDGExperts = nil;
  EditMenu, OptionMenu: TMenuItem;
  MainMenu: TMainMenu;
  SourceEditorNotifiers: TList = nil;

  DGIDENotifier: TDgIDENotifier;
  Services: IOTAServices = nil;
  IDENotifierIndex: Integer = -1;

implementation

uses DGUtils;

procedure ClearSourceEditorNotifiers;
var
  I: Integer;
begin
  if Assigned(SourceEditorNotifiers) then
    for I := SourceEditorNotifiers.Count - 1 downto 0 do
      // Destroyed calls RemoveNotifier which in turn releases the instance
      TDgSourceEditorNotifier(SourceEditorNotifiers[I]).Destroyed;
end;

procedure InstallSourceEditorNotifiers(Module: IOTAModule);
var
  I: Integer;
  SourceEditor: IOTASourceEditor;
begin
  for I := 0 to Module.ModuleFileCount - 1 do
    if Supports(Module.ModuleFileEditors[I], IOTASourceEditor, SourceEditor) then
    begin
      SourceEditorNotifiers.Add(TDgSourceEditorNotifier.Create(SourceEditor));
      SourceEditor := nil;
    end;
end;

{ TDgSourceEditorNotifier }

constructor TDgSourceEditorNotifier.Create(AEditor: IOTASourceEditor);
begin
  inherited Create;
  FEditor := AEditor;
  FIndex := FEditor.AddNotifier(Self);
end;

destructor TDgSourceEditorNotifier.Destroy;
begin
  SourceEditorNotifiers.Remove(Self);
  FEditor := nil;
  inherited Destroy;
end;

procedure TDgSourceEditorNotifier.Destroyed;
begin
  //  invoke MainForm Event Handler
  try
    Form.StopCallback;
    FEditor.RemoveNotifier(FIndex);
  except
  end;
end;

procedure TDgSourceEditorNotifier.ViewActivated(const View: IOTAEditView);
begin
  //  invoke MainForm Event Handler
  try
    Form.StartCallback;
  except
  end;
end;

procedure TDgSourceEditorNotifier.ViewNotification(const View: IOTAEditView; Operation: TOperation);
begin
  //  invoke MainForm Event Handler
  try
    if Operation = opRemove then
      Form.StopCallback
    else
      Form.StartCallback;
  except
  end;
end;

{ TDgExpert }

constructor TDGExperts.Create;
begin
  inherited;
  Form := TfrmOptions.Create(nil);
end;

destructor TDGExperts.Destroy;
begin
  try
    FreeAndNil(Form);
  except
    Debug('TDGExperts.Destory Exception ...');
    Debug;
  end;
  inherited;
end;

procedure TDGExperts.OnOptionClickMenu(Sender: TObject);
begin
  try
    if Assigned(Form) then Form.Show;
  except
    Debug('TDGExperts.OnOptionClickMenu Exception ...');
    Debug;
  end;
end;

{ TDgIDENotifier }

procedure TDGIDENotifier.AfterCompile(Succeeded: Boolean);
begin
  //
end;

procedure TDGIDENotifier.BeforeCompile(const Project: IOTAProject;
  var Cancel: Boolean);
begin
  //
end;

procedure TDGIDENotifier.FileNotification(NotifyCode: TOTAFileNotification;
  const FileName: string; var Cancel: Boolean);
begin
  Debug('Notification: ' + IntToStr(Integer(NotifyCode)), FileName);
  try
    case NotifyCode of
      ofnFileOpened: Form.StartCallback;
      ofnFileClosing: Form.StopCallback;
    end;
  except
    Debug('TDGIDENotifier.FileNotification Exception ...');
    Debug;
  end;
end;

procedure Register;
var
  i: Integer;
  NTAServices: INTAServices;
{$IFDEF ENABLE_EDITOR_NOTIFIER}
  ModuleServices: IOTAModuleServices;
  EditorServices: IOTAEditorServices;
  EditorTopView: IOTAEditView;
  j: Integer;
{$ENDIF}
begin
  DGExperts := TDGExperts.Create();

  // install the MenuItem under Delphi/Edit Menu
  OptionMenu := TMenuItem.Create(Application);
  OptionMenu.Caption := EXPERT_MENU_CAPTION;
  // OptionMenu.ShortCut := ShortCut(Ord('D'), [ssCtrl]);  // Ctrl+D
  OptionMenu.OnClick := DgExperts.OnOptionClickMenu;

  // Find Delphi 'Edit' Menu
  NTAServices := (BorlandIDEServices as INTAServices);
  EditMenu := nil;
  MainMenu := nil;
  if Assigned(NTAServices) then
  begin
    MainMenu := NTAServices.MainMenu;
    for i := 0 to MainMenu.Items.Count - 1 do
    begin
      if MainMenu.Items[i].Caption = 'Edit' then
      begin
        EditMenu := MainMenu.Items[i];
        Break;
      end;
    end;

    if not Assigned(EditMenu) then
      if Assigned(MainMenu) then
        EditMenu := MainMenu.Items[1];

    if Assigned(EditMenu) then //  Last Position
      EditMenu.Insert(EditMenu.Count, OptionMenu);
  end;
  NTAServices := nil;
  MainMenu := nil;
  EditMenu := nil;

{$IFDEF ENABLE_IDE_NOTIFIER}
  // install IDE notifier so that we can install editor notifiers for any newly opened module
  Services := BorlandIDEServices as IOTAServices;
  DGIDENotifier := TDGIDENotifier.Create;
  IDENotifierIndex := Services.AddNotifier(DGIDENotifier);
  Services := nil;
{$ENDIF}

{$IFDEF ENABLE_EDITOR_NOTIFIER}
  //  Install the SourceEditorNotifyer
  SourceEditorNotifiers := TList.Create;
  ModuleServices := BorlandIDEServices as IOTAModuleServices;
  if Assigned(ModuleServices) and (ModuleServices.ModuleCount > 0) then
  begin
    for i := 0 to ModuleServices.ModuleCount - 1 do
      InstallSourceEditorNotifiers(ModuleServices.Modules[I]);

    // hook currently active module
    EditorServices := BorlandIDEServices as IOTAEditorServices;
    if Assigned(EditorServices) then
    begin
      EditorTopView := EditorServices.TopView;
      if Assigned(EditorTopView) then
      begin
        for i := 0 to SourceEditorNotifiers.Count - 1 do
        begin
          with TDgSourceEditorNotifier(SourceEditorNotifiers[I]) do
          begin
            for j := 0 to FEditor.EditViewCount - 1 do
            begin
              if FEditor.EditViews[J] = EditorTopView then
              begin
                ViewActivated(EditorTopView);
                Exit;
              end
            end
          end;
        end;
      end;
    end;
  end;
  ModuleServices := nil;
  EditorServices := nil;
  EditorTopView := nil;
{$ENDIF}

end;

initialization

finalization
  //  Auto-Free
{$IFDEF ENABLE_IDE_NOTIFIER}
  try
    Services := BorlandIDEServices as IOTAServices;
    Services.RemoveNotifier(IDENotifierIndex);
    Services := nil;
    FreeAndNil(DGIDENotifier);
  except
    Debug;
  end;
  DgIDENotifier := nil;
{$ENDIF}

{$IFDEF ENABLE_EDITOR_NOTIFIER}
  try
    ClearSourceEditorNotifiers;
  except
  end;
  FreeAndNil(SourceEditorNotifiers);
{$ENDIF}

  try
    OptionMenu.Free;
  except
    Debug;
  end;

  try
    FreeAndNil(DgExperts);
  except
    Debug;
  end;
end.

